/**
 * {}
 * @author yutent<yutent.io@gmail.com>
 * @date 2023/03/06 12:08:35
 */

const boolMap = Object.create(null)

;[
  'autofocus',
  'autoplay',
  'async',
  'allowTransparency',
  'checked',
  'controls',
  'declare',
  'disabled',
  'defer',
  'defaultChecked',
  'defaultSelected',
  'contentEditable',
  'isMap',
  'loop',
  'multiple',
  'noHref',
  'noResize',
  'noShade',
  'open',
  'readOnly',
  'selected',
  'loading'
].forEach(function (name) {
  boolMap[name.toLowerCase()] = name
})

export { boolMap }

export const KEEP_ALIVE = 'keep-alive' // hyphen
export const KEEP_ALIVE_C = 'keepAlive' // camelize
export const WC_PART = Symbol('wc_path')
export const NOTHING = Symbol('wc-nothing')
export const __finalized__ = Symbol('finalized')
export const __props__ = Symbol('props')
export const __changed_props__ = Symbol('changed_props')
export const __mounted__ = Symbol('mounted')
export const __pending__ = Symbol('pending')

export const RESET_CSS_STYLE = `* {box-sizing: border-box;margin: 0;padding: 0;}::before,::after {box-sizing: border-box;}::selection {background: var(--selection-background, var(--color-plain-a));color: var(--selection-color, inherit);}`

function getDefaultValue(type) {
  switch (type) {
    case Number:
      return 0

    case Boolean:
      return false

    case Object:
      return {}

    case Array:
      return []

    default:
      return ''
  }
}

function getType(v) {
  switch (typeof v) {
    case 'number':
      return { type: Number, default: v }

    case 'boolean':
      return { type: Boolean, default: v }

    case 'object':
      return Array.isArray(v)
        ? { type: Array, default: v }
        : { type: Object, default: v }

    case 'string':
      let type = String
      let attribute = true
      if (v.includes('!')) {
        if (v.startsWith('str!')) {
          v = v.slice(4)
        } else if (v.startsWith('num!')) {
          type = Number
          v = +v.slice(4) || 0
        } else if (v.startsWith('bool!')) {
          type = Boolean
          v = v.slice(5)
          v = v !== 'false' && v !== ''
        }
        attribute = false
      }
      return { type, default: v, attribute }

    default:
      return { type: null, default: '' }
  }
}

export function parsePropsDeclaration(options) {
  if (options && typeof options === 'object') {
    if (options.hasOwnProperty('type')) {
      return Object.assign(
        { attribute: true, default: getDefaultValue(options.type) },
        options
      )
    }
  }
  switch (options) {
    case Number:
      options = { type: Number }
      break

    case Boolean:
      options = { type: Boolean }
      break

    case Object:
      options = { type: Object }
      break

    case Array:
      options = { type: Array }
      break

    case String:
      options = { type: String }
      break

    default:
      options = getType(options)
      break
  }
  options.default = options.hasOwnProperty('default')
    ? options.default
    : getDefaultValue(options.type)
  options.attribute = options.hasOwnProperty('attribute')
    ? options.attribute
    : true
  return options
}

export function fixedValue(value, options) {
  switch (options.type) {
    case Number:
      return value === null ? null : +value || 0

    case Boolean:
      return value === false ? false : value !== null

    case Object:
      if (typeof value === 'object') {
        return value
      } else {
        try {
          return JSON.parse(value)
        } catch (err) {
          return {}
        }
      }

    case Array:
      if (typeof value === 'object') {
        return value
      } else {
        try {
          return JSON.parse(value)
        } catch (err) {
          return []
        }
      }

    case null:
      return value

    default:
      return value === null || value === void 0 ? null : value + ''
  }
}
